#include <stdint.h>

#include "xparameters.h"
#include "ipcore_data_capture.h"

#define	REGS		((volatile uint32_t*)XPAR_AXI_STREAMCAPTURE_0_S_AXI_BASEADDR)

#define	NSAMPLES	32u


enum {
	/// Control bits:
	/// 0 - Interrupt status
	REGISTERINDEX_CONTROL = 0,

	/// Transfer size
	REGISTERINDEX_TRANSFER_SIZE = 1,

	/// Timestamp(31:0)
	REGISTERINDEX_TRANSFER_COUNT = 2,

	/// Memory buffer base address
	REGISTERINDEX_ADDRESS = 3,
};

enum {
	/// 0=>1 triggers.
	BV_CONTROL_SOFTTRIGGER = 1 << 0,

	/// Tell the internal FIFO to hold the data instead of just ignoring it.
	/// This has to be set for the duration of the data transfer.
	BV_CONTROL_DATAHOLD = 1 << 1,
};


/// Last transfer count.
static unsigned int _last_transfer_count = 0;

#if (0)
/// Samples buffer.
static uint64_t	buffer[NSAMPLES];
#endif

#if (0)
#define	buffer	((volatile uint64_t*)0x1F000000)
#endif

#if (1)
static volatile uint64_t	real_buffer[NSAMPLES + 4096];
/// Samples buffer.
// #define	buffer				((volatile uint64_t*)(4096u * (((uintptr_t)real_buffer + 4096u - 1u) / 4096u)))
static volatile uint64_t*	buffer = (volatile uint64_t*)0x1F000000;
#endif

// --------------------------------------------------------------------------
void init_data_capture(void)
{
	buffer = (volatile uint64_t*)(4096u * (((uintptr_t)(real_buffer) + 4096u - 1u) / 4096u));
	REGS[REGISTERINDEX_CONTROL] = 0;
	REGS[REGISTERINDEX_ADDRESS] = (uint32_t)buffer;
	REGS[REGISTERINDEX_TRANSFER_SIZE] = NSAMPLES * sizeof(uint64_t);
	_last_transfer_count = REGS[REGISTERINDEX_TRANSFER_COUNT];
}

// --------------------------------------------------------------------------
void start_data_capture(void)
{
	// Transfer has to be disabled for a moment, otherwise the trigger won't work.
	REGS[REGISTERINDEX_CONTROL] = BV_CONTROL_DATAHOLD;

	// Start the trigger sequence.
	REGS[REGISTERINDEX_CONTROL] = BV_CONTROL_SOFTTRIGGER | BV_CONTROL_DATAHOLD;
}

// --------------------------------------------------------------------------
bool is_data_capture_in_progress(void)
{
	const uint32_t	new_transfer_count = REGS[REGISTERINDEX_TRANSFER_COUNT];
	if (new_transfer_count == _last_transfer_count) {
		const uint32_t	control = REGS[REGISTERINDEX_CONTROL];
		if ((control & BV_CONTROL_SOFTTRIGGER) != 0u) {
			return true;
		}
	}
	else {
		/// Completed.
		REGS[REGISTERINDEX_CONTROL] = 0;
	}
	return false;
}

// --------------------------------------------------------------------------
int data_capture_result(const unsigned int channel)
{
	return (int16_t)((buffer[0] >> (channel*16)) & 0xFFFF);
}
