/// \file ipcore_foc.c
/// \brief IP core "FOC"

#pragma once

#include <stdint.h>

#if defined(__cplusplus)
extern "C"
#endif

/// BTN0
#define	BTN0	(1u << 8)

/// BTN1
#define	BTN1	(1u << 9)

/// BTN2
#define	BTN2	(1u << 10)

/// BTN3
#define	BTN3	(1u << 11)


/// Call this first.
extern void init_foc(void);

/// Set FOC mode.
extern void foc_write_mode(const unsigned int mode);

/// Get FOC mode.
extern unsigned int foc_read_mode(void);

/// Write fixed speed delay.
void foc_write_fixperiod(unsigned int delay);

/// Initialize the motor.
extern void foc_init_motor(void);

/// Set speed.
extern void foc_write_speed(int speed_rpm);

/// Read the buttons register.
extern uint32_t read_foc_buttons(void);

/// Read speed, in RPM.
int read_foc_speed(void);

/// Read Id, in arbitrary units.
int read_foc_id(void);

/// Read Iq, in arbitrary units.
int read_foc_iq(void);

/// Print all FOC parameters.
void print_foc_parameters(void);


#if defined(__cplusplus)
}
#endif
